@@ -8,6 +8,7 @@ from ui.Toolbar import Toolbar
 from ui.layout import Arrangement
 from ui.layout import HBox, VBox
 from ui.Pixmap import Pixmap
+from mediabox import media_bookmarks
 from mediabox import tagreader
 from utils import imageloader
 from utils import logging
@@ -19,7 +20,8 @@ import gobject
 _LANDSCAPE_ARRANGEMENT = """
   <arrangement>
     <widget name="btn_nav" x1="0" y1="-64" x2="64" y2="100%"/>
-    <widget name="progress" x1="90" y1="-50" x2="-90" y2="-10"/>
+    <widget name="progress" x1="90" y1="-50" x2="-154" y2="-10"/>
+    <widget name="btn_star" x1="-154" y1="-64" x2="-90" y2="100%"/>
     <widget name="toolbar" x1="-80" y1="0" x2="100%" y2="100%"/>
     <widget name="slider" x1="0" y1="0" x2="40" y2="-64"/>
     
@@ -59,6 +61,8 @@ class AudioPlayer(Player):
 
         self.__offscreen_buffer = None
         
+        # the currently playing file object (e.g. used for bookmarking)
+        self.__current_file = None
     
         Player.__init__(self)
         self.set_visible(False)
@@ -101,6 +105,12 @@ class AudioPlayer(Player):
         # progress bar
         self.__progress = ProgressBar()
         self.__progress.connect_changed(self.__on_seek)
+        self.__progress.connect_bookmark_changed(self.__on_change_bookmark)
+
+        # star button for bookmarks
+        self.__btn_star = ImageButton(theme.mb_btn_bookmark_1,
+                                      theme.mb_btn_bookmark_2)
+        self.__btn_star.connect_clicked(self.__on_btn_star)
 
 
         # toolbar elements
@@ -128,6 +138,7 @@ class AudioPlayer(Player):
         self.__arr.add(self.__btn_navigator, "btn_nav")
         self.__arr.add(self.__toolbar, "toolbar")
         self.__arr.add(self.__progress, "progress")
+        self.__arr.add(self.__btn_star, "btn_star")
         self.__arr.add(self.__volume_slider, "slider")
         self.__arr.add(self.__cover_art, "cover")
         #self.__arr.add(self.__lbl_title, "lbl_title")
@@ -152,12 +163,14 @@ class AudioPlayer(Player):
     
         w, h = self.get_size()
         if (w < h):
+            self.__btn_star.set_visible(False)
             self.__arr.set_xml(_PORTRAIT_ARRANGEMENT)           
             self.__lbl_title.set_alignment(Label.CENTERED)
             self.__lbl_album.set_alignment(Label.CENTERED)
             self.__lbl_artist.set_alignment(Label.CENTERED)
 
         else:
+            self.__btn_star.set_visible(True)
             self.__arr.set_xml(_LANDSCAPE_ARRANGEMENT)
             self.__lbl_title.set_alignment(Label.LEFT)
             self.__lbl_album.set_alignment(Label.LEFT)
@@ -181,6 +194,12 @@ class AudioPlayer(Player):
         self.emit_message(msgs.MEDIA_ACT_PREVIOUS)
 
 
+    def __on_btn_star(self):
+    
+        print "ADDING BOOKMARK"
+        self.__progress.add_bookmark()
+
+
     def __on_btn_next(self):
         
         self.__sliding_direction = self.SLIDE_LEFT
@@ -218,11 +237,17 @@ class AudioPlayer(Player):
                 self.__progress.set_message("Buffering")
 
 
+    def __on_change_bookmark(self):
+    
+        media_bookmarks.set_bookmarks(self.__current_file,
+                                      self.__progress.get_bookmarks())
+
+
+
     def __on_error(self, ctx_id, err):
     
-        if (ctx_id == self.__context_id):
-            # TODO: display readable text here...
-            self.emit_message(msgs.UI_ACT_SHOW_INFO, `err`)
+        if (ctx_id == self.__context_id and self.__player):
+            self.emit_message(msgs.UI_ACT_SHOW_INFO, self.__player.ERRORS[err])
 
 
     def __on_seek(self, progress):
@@ -254,10 +279,10 @@ class AudioPlayer(Player):
                 self.emit_message(msgs.MEDIA_EV_TAG, "TITLE", title)
             if (album):
                 self.__lbl_album.set_text(album)
-                self.emit_message(msgs.MEDIA_EV_TAG, "ALBUM", title)
+                self.emit_message(msgs.MEDIA_EV_TAG, "ALBUM", album)
             if (artist):
                 self.__lbl_artist.set_text(artist)
-                self.emit_message(msgs.MEDIA_EV_TAG, "ARTIST", title)
+                self.emit_message(msgs.MEDIA_EV_TAG, "ARTIST", artist)
             if (cover):
                 imageloader.load_data(cover, self.__on_loaded_cover,
                                       self.__context_id)
@@ -299,6 +324,10 @@ class AudioPlayer(Player):
         self.__lbl_title.set_text(title)
         self.__lbl_artist.set_text(artist)
         self.__lbl_album.set_text(album)
+        
+        self.emit_message(msgs.MEDIA_EV_TAG, "TITLE", title)
+        self.emit_message(msgs.MEDIA_EV_TAG, "ARTIST", artist)
+        self.emit_message(msgs.MEDIA_EV_TAG, "ALBUM", album)
 
         if (self.__offscreen_buffer):
             self.render_buffered(self.__offscreen_buffer)
@@ -347,8 +376,12 @@ class AudioPlayer(Player):
             logging.error("error loading media file: %s\n%s",
                           f, logging.stacktrace())
 
+        self.__current_file = f
         self.__load_track_info(f)
 
+        # load bookmarks
+        self.__progress.set_bookmarks(media_bookmarks.get_bookmarks(f))
+
         self.emit_message(msgs.MEDIA_EV_LOADED, self, f)
 
         #self.__need_slide_in = True
@@ -362,25 +395,25 @@ class AudioPlayer(Player):
 
     def handle_MEDIA_ACT_PLAY(self):
     
-        if (self.__player and self.is_enabled()):
+        if (self.__player and self.is_visible()):
             self.__player.play()
 
 
     def handle_MEDIA_ACT_PAUSE(self):
     
-        if (self.__player and self.is_enabled()):
+        if (self.__player and self.is_visible()):
             self.__player.pause()
 
 
     def handle_MEDIA_ACT_STOP(self):
     
-        if (self.__player and self.is_enabled()):
+        if (self.__player and self.is_visible()):
             self.__player.stop()
 
 
     def handle_INPUT_EV_VOLUME_UP(self, pressed):
     
-        if (self.is_enabled()):
+        if (self.is_visible()):
             self.__volume = min(100, self.__volume + 5)
             self.__volume_slider.set_value(self.__volume / 100.0)
             if (self.__player):
@@ -389,7 +422,7 @@ class AudioPlayer(Player):
         
     def handle_INPUT_EV_VOLUME_DOWN(self, pressed):
 
-        if (self.is_enabled()):    
+        if (self.is_visible()):    
             self.__volume = max(0, self.__volume - 5)
             self.__volume_slider.set_value(self.__volume / 100.0)
             if (self.__player):
@@ -398,6 +431,12 @@ class AudioPlayer(Player):
 
     def handle_INPUT_EV_PLAY(self, pressed):
 
-        if (self.is_enabled() and self.__player):
+        if (self.is_visible() and self.__player):
             self.__player.pause()
 
+
+    def handle_COM_EV_APP_SHUTDOWN(self):
+    
+        if (self.__player):
+            self.__player.close()
+
